/*
 * Simple packet capturing and network monitoring tool.
 * 
 * nsniff.h - main program header.
 *
 * Written by Alexander Hauser
 * lex@lufix.net
 * http://www.lufix.net/
 * http://www.sac.cc/
 *
 * */

#define VERSION 0.1.2

#include "config.h"

#include <sys/types.h>

#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#include <pcap.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif

/* protocol specific headers */
#include "ip.h"
#include "ip6.h"
#include "icmp6.h"
#include "udp.h"
#include "tcp.h"
#include "ip_icmp.h"
#include "ether.h"
#include "dns.h"

#define SNAPLEN 1024
#define ETH_ALEN 6

/* port definitions */
#define PORT_DNS	53

struct sockaddr localaddr;
socklen_t	locallen;

int datalink,		/* from pcap_datalink() */
	snaplen,		/* amount of data to capture */
	compile,		/* is set when user gives an bpf-code to compile */
	verbose,		/* is set when -v option specified */
	names,			/* is set when -n option specified */
	etherp,			/* is set when -e option specified */
	counto,			/* defines how much packets to sniff (-c option) */
	count,			/* used for counting packets when -c option is set */
	procraw;		/* is set when -r option specified */

char *device,		/* pcap device */
	 *compstr;		/* bpf-code */

int enet_hdr;

pcap_t *pd;							/* pcap descriptor */
pcap_dumper_t *pdt;

char *copy_argv(char **);			/* extracts the bpf-code out of argv */
void open_pcap();					/* opens a pcap device and compiles in
										the bpf-code if specified */
void readloop();					/* main reading-loop */
void print_etheradrs(uint8_t *, uint8_t *);	
									/* prints MAC addresses */
void print_ip6addr(const char *);	/* printd IPv6 addresses */
void print_time(struct timeval *);	/* prints precise time */
void checkcount();					/* checks if specified count of
										captured packets has been reached
										(-c option) */
void checkport(uint16_t, uint16_t, char *);
									/* checks if the port belongs to a
										known application-layer-protocol */
void cleanup();						/* prints a goodbye banner and exits */
void usage(const char *);			/* prints usage information */
void banner();						/* prints an information banner */

void *ip_process(char *, int);		/* IP header processing */
void *ip6_process(char *, int);		/* IPv6 header processing */

void *ip6hopopts_process(char *);
void *ip6dstopts_process(char *);
void *ip6rout_process(char *);
void *ip6frag_process(char *);

/* protocol specific functions */
void *tcp_process(char *, int);		/* TCP header processing */
void udp_process(char *);			/* UDP header processing */
void icmp_process(char *);			/* ICMP processing */
void icmp6_process(char *);			/* ICMPv6 processing */
void arp_process(char *, uint8_t sender[ETH_ALEN], uint8_t target[ETH_ALEN]);
									/* ARP processing */

void scanEtherProt(uint16_t);		/* determines other transpor-layer
									   protocols */

void dns_process(char *);			/* DNS processing */
