/*
 * Simple packet capturing and network monitoring tool.
 * 
 * proc_icmp.c - function for processing the ICMP protocol.
 *
 * Written by Alexander Hauser
 * lex@lufix.net
 * http://www.lufix.net/
 * http://www.sac.cc/
 *
 * */

#include "nsniff.h"

void icmp_process(char *ptr) {
	struct icmp *icmp;
		
	icmp = (struct icmp *) ptr;
	printf("\nProtocol: ICMP.");
	switch (icmp->icmp_type) {
		case ICMP_ECHOREPLY:
			printf(" Echo-reply (type %d).", icmp->icmp_type);
			break;
		case ICMP_DEST_UNREACH:
			printf(" Destination-unreachable (type %d):", icmp->icmp_type);
			switch (icmp->icmp_code) {
				case ICMP_NET_UNREACH:
					printf("network unreachable (code %d).", icmp->icmp_code);
					break;
				case ICMP_HOST_UNREACH:
					printf("host unreachable (code %d).", icmp->icmp_code);
					break;
				case ICMP_PROT_UNREACH:
					printf("protocol unreachable (code %d).", icmp->icmp_code);
					break;
				case ICMP_PORT_UNREACH:
					printf("port unreachable (code %d).", icmp->icmp_code);
					break;
				case ICMP_FRAG_NEEDED:
					printf("fragmentation needed (code %d).", icmp->icmp_code);
					break;
				case ICMP_SR_FAILED:
					printf("source-route failed (code %d).", icmp->icmp_code);
					break;
				case ICMP_NET_UNKNOWN:
					printf("unknown network (code %d).", icmp->icmp_code);
					break;
				case ICMP_HOST_UNKNOWN:
					printf("unknown host (code %d).", icmp->icmp_code);
					break;
				case ICMP_HOST_ISOLATED:
					printf("host isolated (code %d).", icmp->icmp_code);
					break;
				case ICMP_NET_ANO:
					break;
				case ICMP_HOST_ANO:
					break;
				case ICMP_NET_UNR_TOS:
					printf("network unreachable for type-of-service (code %d).", icmp->icmp_code);
					break;
				case ICMP_HOST_UNR_TOS:
					printf("host unreachable for type-of-service (code %d).", icmp->icmp_code);
					break;
				case ICMP_PKT_FILTERED:
					printf("packet filtered (code %d).", icmp->icmp_code);
					break;
				case ICMP_PREC_VIOLATION:
					printf("precedence violation (code %d).", icmp->icmp_code);
					break;
				case ICMP_PREC_CUTOFF:
					printf("precedence cutoff (code %d).", icmp->icmp_code);
					break;
			}
			break;
		case ICMP_SOURCE_QUENCH:
			printf(" Source quench  (type %d).", icmp->icmp_type);
			break;
		case ICMP_REDIRECT:
			printf("redirect (type %d) for: ", icmp->icmp_type);
			switch (icmp->icmp_code) {
				case ICMP_REDIR_NET:
					printf("network (code %d).", icmp->icmp_code);
					break;
				case ICMP_REDIR_HOST:
					printf("host (code %d).", icmp->icmp_code);
					break;
				case ICMP_REDIR_NETTOS:
					printf("network for type-of-service (code %d).", icmp->icmp_code);
					break;
				case ICMP_REDIR_HOSTTOS:
					printf("host for type-of-service (code %d).", icmp->icmp_code);
					break;
			}
			break;
		case ICMP_ECHO:
			printf(" Echo-request (type %d).", icmp->icmp_type);
			break;
		case ICMP_ROUTERSOLICIT:
			printf(" Router-Solicitation (type %d).", icmp->icmp_type);
			break;
		case ICMP_ROUTERADVERT:
			printf(" Router-Advertisement (type %d).", icmp->icmp_type);
			break;
		case ICMP_TIME_EXCEEDED:
			printf(" Time-exceeded (type %d):", icmp->icmp_type);
			switch (icmp->icmp_code) {
				case ICMP_EXC_TTL:
					printf("time-to-live count exceeded.");
					break;
				case ICMP_EXC_FRAGTIME:
					printf("fragment reass time exceeded.");
					break;
			}
			break;
		case ICMP_PARAMETERPROB:
			printf(" Parameter-problem (type %d).", icmp->icmp_type);
			break;
		case ICMP_TIMESTAMP:
			printf(" Timestamp-request (type %d).", icmp->icmp_type);
			break;
		case ICMP_TIMESTAMPREPLY:
			printf(" Timestamp-reply (type %d).", icmp->icmp_type);
			break;
		case ICMP_INFO_REQUEST:
			printf(" Info-request (type %d).", icmp->icmp_type);
			break;
		case ICMP_INFO_REPLY:
			printf(" Info-reply (type %d).", icmp->icmp_type);
			break;
		case ICMP_ADDRESS:
			printf(" Address-request (type %d).", icmp->icmp_type);
			break;
		case ICMP_ADDRESSREPLY:
			printf(" Address-reply (type %d).", icmp->icmp_type);
			printf(" Address mask is 0x%08x.", ntohl(icmp->icmp_mask));
			break;
	}
	if (verbose)
		printf(" ICMP-ID: %d; ICMP SEQ: %d.", icmp->icmp_id, icmp->icmp_seq);
	 
	return;
}
