/*
 * Simple packet capturing and network monitoring tool.
 * 
 * proc_ip.c - function for processing the IP header.
 *
 * Written by Alexander Hauser
 * lex@lufix.net
 * http://www.lufix.net/
 * http://www.sac.cc/
 *
 * */

#include "nsniff.h"

/* processes the IP header */
void *ip_process(char *ptr, int len) {
	int hlen;
	struct ip *ip;
	struct hostent *hptr;
		
	printf("IP-packet cached. ");
	if (len < sizeof(struct ip)) {
		printf("hmm, looks to small for an IP packet. skipping...");
		return;
	}
	ip = (struct ip *) ptr;
	if (ip->ip_v != 4) {
		printf("...but IP version (%d) != 4. skipping...", ip->ip_v);
		return;
	}
	hlen = ip->ip_hl << 2;
	if (hlen < sizeof(struct ip)) {
		printf("hmm, the header is to small (%d) for an IP packet. skipping...\n", hlen);
		return;
	}		
	if (verbose)
		printf("IP-total-length is %d. ", ntohs(ip->ip_len));
	if (names) {
		if ((hptr = gethostbyaddr(&ip->ip_src, 4, AF_INET)) == NULL)
			printf("%s >>", inet_ntoa(ip->ip_src));
		else
			printf("%s >>", hptr->h_name);
		if ((hptr = gethostbyaddr(&ip->ip_dst, 4, AF_INET)) == NULL)
			printf(" %s.", inet_ntoa(ip->ip_dst));
		else
			printf(" %s .", hptr->h_name);
	} else {
		printf("%s >>", inet_ntoa(ip->ip_src));
		printf(" %s.", inet_ntoa(ip->ip_dst));
	}
	if (verbose) {
		printf(" TTL: %d, ID: %d,", ip->ip_ttl, ip->ip_id);
		switch (ip->ip_tos) {
			case IPTOS_TOS_MASK:
				printf(" TOS: mask");
				break;
			case IPTOS_LOWDELAY:
				printf(" TOS: LowDelay");
				break;
			case IPTOS_THROUGHPUT:
				printf(" TOS: MaxThroughput");
				break;
			case IPTOS_RELIABILITY:
				printf(" TOS: MaxReliability");
				break;
			case IPTOS_LOWCOST:
				printf(" TOS: LowCost");
				break;
			default:
				printf(" TOS not specified");
				break;
		}
	}
		
	switch (ip->ip_p) {
		case IPPROTO_TCP:
			tcp_process(ptr + hlen, len-hlen);
			break;
		case IPPROTO_UDP:
			udp_process(ptr + hlen);
			break;
		case IPPROTO_ICMP:
			icmp_process(ptr + hlen);
			break;
		default:
			printf("packet contains an unknown protocol. skipping...");
			break;
	}
	
	return;
}

